/*----------------------------------------------------------------------
  File    : winskel.c
  Contents: windows program skeleton
  Author  : Christian Borgelt
  History : 03.03.1998 file created from skeleton.c
            09.11.1999 access of resource string table added
            15.09.2000 some type casts added for C++ compatibility
----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#include <commdlg.h>
#include "winskel.rh"

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define PRGNAME    "WinSkel"    /* main window title */

/* --- abbreviations --- */
#define HINST      HINSTANCE
#define MKIRSC     MAKEINTRESOURCE
#define SDIMSG     SendDlgItemMessage
#define WINSTYLE   WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN

/* --- extensions --- */
#define INITXEXT   320          /* initial horizontal extension */
#define INITYEXT   240          /* initial vertical   extension */

/* --- edit box functions --- */
#define edt_set(h,i,s)    SetDlgItemText(h, i, s)
#define edt_get(h,i,s,n)  GetDlgItemText(h, i, s, n)
#define int_set(h,i,n)    SetDlgItemInt(h, i, n, 0)
#define int_get(h,i)      GetDlgItemInt(h, i, NULL, 0)

/* --- check box functions --- */
#define chk_set(h,i,s)    CheckDlgButton(h, i, s)
#define chk_get(h,i)      IsDlgButtonChecked(h, i)

/* --- combo box functions --- */
#define cmb_add(h,i,s)    SDIMSG(h, i, CB_ADDSTRING,    0, (LPARAM)(s))
#define cmb_rem(h,i,k)    SDIMSG(h, i, CB_DELETESTRING, k, 0)
#define cmb_set(h,i,k)    SDIMSG(h, i, CB_SETCURSEL,    k, 0)
#define cmb_get(h,i)      SDIMSG(h, i, CB_GETCURSEL,    0, 0)

/*----------------------------------------------------------------------
  Global Variables
----------------------------------------------------------------------*/
static HINST hinst = NULL;      /* instance handle */
static HWND  hmain = NULL;      /* handle of main window */
static char  buf[1024];         /* text buffer */
static struct {                 /* parameter block */
  char text[81];                /* a text */
  int  number;                  /* a number */
  int  selection;               /* a selection from a combobox */
  int  check;                   /* a checkbox state */
  char filename[_MAX_PATH];     /* a file name */
} params = {                    /* initial values */
  "abc", 123, 0, 0, "noname.dat" };

/* --- pens --- */
static HPEN    pen_black;       /* black  : lines */
static HPEN    pen_white;       /* white  : lines */
static HPEN    pen_red;         /* yellow : circles */
static HPEN    pen_green;       /* green  : circles */
static HPEN    pen_blue;        /* blue   : circles */
static HPEN    pen_yellow;      /* yellow : circles */

/* --- brushes --- */
static HBRUSH  br_null;         /* hollow brush */
static HBRUSH  br_white;        /* white  : background */
static HBRUSH  br_black;        /* black  : arrow heads */
static HBRUSH  br_red;          /* yellow : circle */
static HBRUSH  br_green;        /* green  : circle */
static HBRUSH  br_blue;         /* blue   : circle */
static HBRUSH  br_yellow;       /* yellow : circle */

/*----------------------------------------------------------------------
  Drawing Function
----------------------------------------------------------------------*/

static void redraw (HDC hdc)
{                               /* --- redraw window contents */
  static int init = 0;          /* initialization flag */
  int  i;                       /* loop variable */
  int  x, y;                    /* coordinates of pixel */
  RECT rc;                      /* rectangle for drawing */
  char *s;                      /* temporary buffer for a string */

  rc.left = 0; rc.right  = INITXEXT -1;
  rc.top  = 0; rc.bottom = INITYEXT -1;
  FillRect(hdc, &rc, br_white);

  SelectObject(hdc, pen_red);    SelectObject(hdc, br_red);
  Ellipse(hdc,  40,  30,  60,  50);
  SelectObject(hdc, pen_blue);   SelectObject(hdc, br_blue);
  Ellipse(hdc,  70,  60,  90,  80);
  SelectObject(hdc, pen_green);  SelectObject(hdc, br_green);
  Ellipse(hdc, 100,  90, 120, 110);
  SelectObject(hdc, pen_yellow); SelectObject(hdc, br_yellow);
  Ellipse(hdc, 130, 120, 150, 140);

  SelectObject(hdc, br_null);
  SelectObject(hdc, pen_red);
  Ellipse(hdc, 130,  30, 149,  49);
  Ellipse(hdc, 133,  33, 146,  46);
  Ellipse(hdc, 136,  36, 143,  43);
  SelectObject(hdc, pen_blue);
  Ellipse(hdc, 160,  60, 179,  79);
  Ellipse(hdc, 163,  63, 176,  76);
  Ellipse(hdc, 166,  66, 173,  73);
  SelectObject(hdc, pen_green);
  Ellipse(hdc, 190,  90, 209, 109);
  Ellipse(hdc, 193,  93, 206, 106);
  Ellipse(hdc, 196,  96, 203, 103);
  SelectObject(hdc, pen_yellow);
  Ellipse(hdc, 220, 120, 239, 139);
  Ellipse(hdc, 223, 123, 236, 136);
  Ellipse(hdc, 226, 126, 233, 133);

  SelectObject(hdc, pen_black);
  MoveToEx(hdc, 10, 25, NULL);
  LineTo(hdc,  60,  25);
  LineTo(hdc, 185, 150);
  LineTo(hdc, 135, 150);
  LineTo(hdc,  10,  25);

  if (!init) { srand((unsigned)time(NULL)); init = 1; }
  for (i = 0; i < 1000; i++) {
    x = (int)(rand()/(double)RAND_MAX *50);
    y = (int)(rand()/(double)RAND_MAX *50);
    SetPixel(hdc, 250 +x, 30 +y, RGB(0xc0, 0xc0, 0xc0));
  }

  SelectObject(hdc, GetStockObject(ANSI_VAR_FONT));
  s = "This is a test.";
  TextOut(hdc, 120, 200, s, strlen(s));
}  /* redraw() */

/*----------------------------------------------------------------------
  Dialog Functions
----------------------------------------------------------------------*/

static const char* fselect (HWND hwnd, char *title, int load)
{                               /* --- select a file name */
  static char fname[_MAX_PATH+1] = "";  /* file name */
  static char path [_MAX_PATH+1];       /* and path */
  static char filter[] =        /* file selection filter */
    "all files (*.*)\0*.*\0";   /* (filter for all files) */
  static OPENFILENAME ofn = {   /* file selector structure */
    sizeof(OPENFILENAME),       /* lStructSize */
    NULL,   NULL,               /* hwndOwner, hInstance */
    filter, NULL,               /* lpstrFilter, lpstrCustomFilter */
    0, 1,                       /* nMaxCustFilter, nFilterIndex */
    fname, _MAX_PATH,           /* lpstrFile, nMaxFile */
    NULL, _MAX_FNAME+_MAX_EXT,  /* lpstrFileTitle, nMaxFileTitle */
    path,                       /* lpstrInitialDir */
    "Select File...",           /* lpstrTitle (with default init.) */
    OFN_PATHMUSTEXIST, 0, 0,    /* Flags, nFileOffset, nFileExtension */
    NULL, 0L,                   /* lpstrDefExt, lCustData */
    NULL, NULL };               /* lpfnHook, lpTemplateName */

  ofn.hwndOwner  = hwnd;        /* set parent window handle */
  ofn.lpstrTitle = title;       /* and dialog box title */
  ofn.Flags = (load) ? OFN_FILEMUSTEXIST : OFN_PATHMUSTEXIST;
  if (GetOpenFileName(&ofn) == 0)
    return NULL;                /* open file selector box */
  return fname;                 /* return name of selected file */
}  /* fselect() */

/*--------------------------------------------------------------------*/

static int load (void)
{                               /* --- File > Load File... */
  const char *fname;            /* name of file to load */

  LoadString(hinst, CS_LOAD, buf, sizeof(buf));
  fname = fselect(hmain, buf,1);/* get the dialog box title */
  if (!fname) return -1;        /* and select a file */
  return 0;                     /* return 'ok' */
}  /* load() */

/*--------------------------------------------------------------------*/

static void save (void)
{                               /* --- File > Save File... */
  const char *fname;            /* name of file to save to */

  LoadString(hinst, CS_SAVE, buf, sizeof(buf));
  fname = fselect(hmain, buf,0);/* get the dialog box title */
  if (!fname) return;           /* and select a file */
}  /* save() */

/*----------------------------------------------------------------------
  Dialog Box Procedures
----------------------------------------------------------------------*/

BOOL CALLBACK dp_about (HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{                               /* --- `about dialog box procedure */ 
  switch (msg) {                /* evaluate message */
    case WM_COMMAND:            /* command (button pressed etc.) */
      if (wp != IDOK) break;    /* process only 'Ok' button */
    case WM_CLOSE:              /* dialog box closed */
      EndDialog(hwnd, 0); return 1;
  }                             /* terminate dialog */
  return 0;                     /* everything else not handled */
}  /* dp_about() */

/*--------------------------------------------------------------------*/

BOOL CALLBACK dp_test (HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{                               /* --- test dialog box procedure */
  int        i;                 /* loop variable */
  const char *fname;            /* buffer for file name */

  switch (msg) {                /* evaluate message */
    case WM_INITDIALOG:         /* dialog initialization */
      edt_set(hwnd, DI_TEXT,   params.text);
      int_set(hwnd, DI_NUMBER, params.number);
      for (i = 0; i < CMBITEMCNT; i++) {
        LoadString(hinst, CS_CMBITEMS +i, buf, sizeof(buf));
        cmb_add(hwnd, DI_COMBO, buf);
      }                         /* set items in combo box */
      cmb_set(hwnd, DI_COMBO, params.selection);
      chk_set(hwnd, DI_CHECK, params.check);
      edt_set(hwnd, DI_FILE,  params.filename);
      return 1;                 /* set defaults from parameter block */

    case WM_COMMAND:            /* command (button pressed etc.) */
      switch (wp) {             /* evaluate command code */
        case DI_BROWSE:         /* if to select a file name */
          LoadString(hinst, CS_BROWSE, buf, sizeof(buf));
          fname = fselect(hwnd, buf, 1);
          if (fname) edt_set(hwnd, DI_FILE, fname);
          return 1;             /* get new file name and abort */
        case IDOK:              /* if 'Ok' button pressed */
          edt_get(hwnd, DI_TEXT, params.text, 80);
          params.number    = int_get(hwnd, DI_NUMBER);
          params.selection = cmb_get(hwnd, DI_COMBO);
          params.check     = chk_get(hwnd, DI_CHECK);
          edt_get(hwnd, DI_FILE, params.filename, _MAX_PATH);
          EndDialog(hwnd, 0);   /* get inputs from edit controls */
          return 1;             /* return 'ok' */
        case IDCANCEL:          /* if 'Cancel' button pressed, */
          EndDialog(hwnd, -1); return 1;   /* terminate dialog */
      } break;

    case WM_CLOSE:              /* dialog box has been closed */
      EndDialog(hwnd, -1); return 1;      /* terminate dialog */
  }
  return 0;                     /* everything else not handled */
}  /* dp_test() */

/*----------------------------------------------------------------------
  Window Procedures
----------------------------------------------------------------------*/

LRESULT CALLBACK wp_main (HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{                               /* --- main window procedure */
  static PAINTSTRUCT ps;        /* for BeginPaint call */
  HDC    hdc;                   /* handle of device context */
  int    x, y;                  /* mouse coordinates */
  HPEN   pen   = NULL;          /* pen for drawing */
  HBRUSH brush = NULL;          /* brush for drawing */

  switch (msg) {                /* evaluate message */
    case WM_PAINT:              /* --- if to paint window */
      hdc = BeginPaint(hwnd, &ps);
      if (!hdc) return 0;       /* get device context for drawing */
      redraw(hdc);              /* redraw the window contents */
      EndPaint(hwnd, &ps);      /* release the device context */
      break;

    case WM_COMMAND:            /* --- if menu item selected */
      switch (wp) {             /* evaluate item */
        /* --- Menu File --- */
        case MI_LOAD:           /* File > Load File... */
          load(); return 0;     /* call dummy load function */

        case MI_SAVE:           /* File > Save File... */
          save(); return 0;     /* call dummy save function */

        case MI_QUIT:           /* File > Quit */
          DestroyWindow(hwnd);  /* destroy main window */
          return 0;             /* and abort the program */

        /* --- Menu Actions --- */
        case MI_REDRAW:         /* Menu Item 'Actions > Redraw' */
          InvalidateRect(hwnd, NULL, 0);
          return 0;             /* invalidate window contents */
        
		case MI_DIALOG:         /* Menu Item 'Actions > Dialog' */
          if (DialogBox(hinst, MKIRSC(DB_TEST), hwnd, dp_test) != 0)
            return 0;           /* open the dialog box */
          hdc = GetDC(hwnd);    /* and check the result */
          TextOut(hdc, 10, 10, params.text, strlen(params.text));
          ReleaseDC(hwnd, hdc); /* print the text entered */
          return 0;             /* in the dialog box */

        /* --- Menu Help --- */
        case MI_ABOUT:          /* Menu Item 'Help > About' */
          DialogBox(hinst, MKIRSC(DB_ABOUT), hwnd, dp_about);
          return 0;             /* show 'about' dialog box */
      } break;

    case WM_LBUTTONDOWN:        /* left mouse button pressed */
      if (!pen) { pen = pen_red;   brush = br_red;   }
    case WM_MBUTTONDOWN:        /* middle mouse button pressed */
      if (!pen) { pen = pen_green; brush = br_green; }
    case WM_RBUTTONDOWN:        /* right mouse button pressed */
      if (!pen) { pen = pen_blue;  brush = br_blue;  }
      hdc = GetDC(hwnd);        /* get device context for drawing */
      if (!hdc) return 0;
      SelectObject(hdc, pen);   /* select a pen and a brush */
      SelectObject(hdc, brush); /* according to the button pressed */
      x = LOWORD(lp); y = HIWORD(lp);
      Ellipse(hdc, x-3, y-3, x+2, y+2);
      ReleaseDC(hwnd, hdc);     /* draw a circle at mouse position */
      break;                    /* and release the device context */

    case WM_CLOSE:              /* window closed */
    case WM_DESTROY:            /* or destroyed */
      PostQuitMessage(0);       /* terminate program */
      return 0;
  }
  return DefWindowProc(hwnd, msg, wp, lp);
}  /* wp_main() */

/*----------------------------------------------------------------------
	Main Program
----------------------------------------------------------------------*/

int WINAPI WinMain (HINSTANCE hcurr, HINSTANCE hprev,
                    LPSTR cmdln, int cmd)
{                               /* --- main function */
  WNDCLASS wc;                  /* window class */
  RECT     rc;                  /* window rectangle */
  MSG      msg;                 /* message */

  /* --- create window class --- */
  hinst = hcurr;                /* note program instance handle */
  if (!hprev) {                 /* if first instance of this program */
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = wp_main;
    wc.cbClsExtra    = wc.cbWndExtra = 0;
    wc.hInstance     = hinst;
    wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName  = MKIRSC(MN_MAIN);
    wc.lpszClassName = "WinSkelClass";
    if (!RegisterClass(&wc))    /* register class of */
      return FALSE;             /* main window */
  }

  /* --- get pens --- */
  pen_white  = (HPEN)GetStockObject(WHITE_PEN);
  pen_black  = (HPEN)GetStockObject(BLACK_PEN);
  pen_red    = CreatePen(PS_SOLID, 1, RGB(0xff, 0x00, 0x00));
  pen_green  = CreatePen(PS_SOLID, 1, RGB(0x00, 0xff, 0x00));
  pen_blue   = CreatePen(PS_SOLID, 1, RGB(0x00, 0x00, 0xff));
  pen_yellow = CreatePen(PS_SOLID, 1, RGB(0xff, 0xff, 0x00));

  /* --- get brushes --- */
  br_null    = (HBRUSH)GetStockObject(HOLLOW_BRUSH);
  br_white   = (HBRUSH)GetStockObject(WHITE_BRUSH);
  br_black   = (HBRUSH)GetStockObject(BLACK_BRUSH);
  br_red     = CreateSolidBrush(RGB(0xff, 0x00, 0x00));
  br_green   = CreateSolidBrush(RGB(0x00, 0xff, 0x00));
  br_blue    = CreateSolidBrush(RGB(0x00, 0x00, 0xff));
  br_yellow  = CreateSolidBrush(RGB(0xff, 0xff, 0x00));

  /* --- create main window --- */
  rc.left = 0; rc.right  = INITXEXT -1;
  rc.top  = 0; rc.bottom = INITYEXT -1;
  AdjustWindowRect(&rc, WINSTYLE, 1);
  hmain = CreateWindow("WinSkelClass", PRGNAME, WINSTYLE,
                       30, 30, rc.right-rc.left, rc.bottom-rc.top,
                       NULL, NULL, hinst, NULL);
  if (hmain == 0) return FALSE; /* create main window */

  /* --- show main window --- */
  ShowWindow(hmain, cmd);       /* open and */
  UpdateWindow(hmain);          /* update main window */

  /* --- dispatch messages --- */
  while (GetMessage(&msg, NULL, 0, 0)) {
    TranslateMessage(&msg);     /* get messages, translate */
    DispatchMessage (&msg);     /* and dispatch them */
  }

  /* --- clean up --- */
  DeleteObject(br_red);         /* delete created brushs */
  DeleteObject(br_green);
  DeleteObject(br_blue);
  DeleteObject(br_yellow);
  DeleteObject(pen_red);        /* delete created pens */
  DeleteObject(pen_green);
  DeleteObject(pen_blue);
  DeleteObject(pen_yellow);
  return msg.wParam;
}  /* WinMain() */
